#!/usr/bin/env python

# Python 2/3 compatibility
from __future__ import print_function

import os, numpy

import cv2 as cv

from tests_common import NewOpenCVTests

class structured_light_test(NewOpenCVTests):

    def test_unwrap(self):
        paramsPsp = cv.structured_light_SinusoidalPattern_Params();
        paramsFtp = cv.structured_light_SinusoidalPattern_Params();
        paramsFaps = cv.structured_light_SinusoidalPattern_Params();
        paramsPsp.methodId = cv.structured_light.PSP;
        paramsFtp.methodId = cv.structured_light.FTP;
        paramsFaps.methodId = cv.structured_light.FAPS;

        sinusPsp = cv.structured_light.SinusoidalPattern_create(paramsPsp)
        sinusFtp = cv.structured_light.SinusoidalPattern_create(paramsFtp)
        sinusFaps = cv.structured_light.SinusoidalPattern_create(paramsFaps)

        captures = []
        for i in range(0,3):
            capture = self.get_sample('/cv/structured_light/data/capture_sin_%d.jpg'%i, cv.IMREAD_GRAYSCALE)
            if capture is None:
                raise unittest.SkipTest("Missing files with test data")
            captures.append(capture)

        rows,cols = captures[0].shape

        unwrappedPhaseMapPspRef = self.get_sample('/cv/structured_light/data/unwrappedPspTest.jpg',
                                                  cv.IMREAD_GRAYSCALE)
        unwrappedPhaseMapFtpRef = self.get_sample('/cv/structured_light/data/unwrappedFtpTest.jpg',
                                                  cv.IMREAD_GRAYSCALE)
        unwrappedPhaseMapFapsRef = self.get_sample('/cv/structured_light/data/unwrappedFapsTest.jpg',
                                                  cv.IMREAD_GRAYSCALE)

        wrappedPhaseMap,shadowMask = sinusPsp.computePhaseMap(captures);
        unwrappedPhaseMap = sinusPsp.unwrapPhaseMap(wrappedPhaseMap, (cols, rows), shadowMask=shadowMask)
        unwrappedPhaseMap8 = unwrappedPhaseMap*1 + 128
        unwrappedPhaseMap8 = numpy.uint8(unwrappedPhaseMap8)

        sumOfDiff = 0
        count = 0
        for i in range(rows):
            for j in range(cols):
                ref = int(unwrappedPhaseMapPspRef[i, j])
                comp = int(unwrappedPhaseMap8[i, j])
                sumOfDiff += (ref - comp)
                count += 1

        ratio = sumOfDiff/float(count)
        self.assertLessEqual(ratio, 0.2)

        wrappedPhaseMap,shadowMask = sinusFtp.computePhaseMap(captures);
        unwrappedPhaseMap = sinusFtp.unwrapPhaseMap(wrappedPhaseMap, (cols, rows), shadowMask=shadowMask)
        unwrappedPhaseMap8 = unwrappedPhaseMap*1 + 128
        unwrappedPhaseMap8 = numpy.uint8(unwrappedPhaseMap8)

        sumOfDiff = 0
        count = 0
        for i in range(rows):
            for j in range(cols):
                ref = int(unwrappedPhaseMapFtpRef[i, j])
                comp = int(unwrappedPhaseMap8[i, j])
                sumOfDiff += (ref - comp)
                count += 1

        ratio = sumOfDiff/float(count)
        self.assertLessEqual(ratio, 0.2)

        wrappedPhaseMap,shadowMask2 = sinusFaps.computePhaseMap(captures);
        unwrappedPhaseMap = sinusFaps.unwrapPhaseMap(wrappedPhaseMap, (cols, rows), shadowMask=shadowMask)
        unwrappedPhaseMap8 = unwrappedPhaseMap*1 + 128
        unwrappedPhaseMap8 = numpy.uint8(unwrappedPhaseMap8)

        sumOfDiff = 0
        count = 0
        for i in range(rows):
            for j in range(cols):
                ref = int(unwrappedPhaseMapFapsRef[i, j])
                comp = int(unwrappedPhaseMap8[i, j])
                sumOfDiff += (ref - comp)
                count += 1

        ratio = sumOfDiff/float(count)
        self.assertLessEqual(ratio, 0.2)

if __name__ == '__main__':
    NewOpenCVTests.bootstrap()



import numpy as np
import cv2 as cv

# aruco config
adict = cv.aruco.Dictionary_get(cv.aruco.DICT_4X4_50)
cv.imshow("marker", cv.aruco.drawMarker(adict, 0, 400))
marker_len = 5

# rapid config
obj_points = np.float32([[-0.5, 0.5, 0], [0.5, 0.5, 0], [0.5, -0.5, 0], [-0.5, -0.5, 0]]) * marker_len
tris = np.int32([[0, 2, 1], [0, 3, 2]])  # note CCW order for culling
line_len = 10

# random calibration data. your mileage may vary.
imsize = (800, 600)
K = cv.getDefaultNewCameraMatrix(np.diag([800, 800, 1]), imsize, True)

# video capture
cap = cv.VideoCapture(0)
cap.set(cv.CAP_PROP_FRAME_WIDTH, imsize[0])
cap.set(cv.CAP_PROP_FRAME_HEIGHT, imsize[1])

rot, trans = None, None
while cv.waitKey(1) != 27:
    img = cap.read()[1]

    # detection with aruco
    if rot is None:
        corners, ids = cv.aruco.detectMarkers(img, adict)[:2]

        if ids is not None:
            rvecs, tvecs = cv.aruco.estimatePoseSingleMarkers(corners, marker_len, K, None)[:2]
            rot, trans = rvecs[0].ravel(), tvecs[0].ravel()

    # tracking and refinement with rapid
    if rot is not None:
        for i in range(5):  # multiple iterations
            ratio, rot, trans = cv.rapid.rapid(img, 40, line_len, obj_points, tris, K, rot, trans)[:3]
            if ratio < 0.8:
                # bad quality, force re-detect
                rot, trans = None, None
                break

    # drawing
    cv.putText(img, "detecting" if rot is None else "tracking", (0, 20), cv.FONT_HERSHEY_SIMPLEX, 1.0, (0, 255, 255))
    if rot is not None:
        cv.drawFrameAxes(img, K, None, rot, trans, marker_len)
    cv.imshow("tracking", img)
